@@ -1,10 +1,10 @@
 package Net::DNS::Domain;
 
 #
-# $Id: Domain.pm 954 2011-11-02 21:32:55Z willem $
+# $Id: Domain.pm 973 2012-01-23 13:33:08Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 954 $)[1];
+$VERSION = (qw$LastChangedRevision: 973 $)[1];
 
 
 =head1 NAME
@@ -41,20 +41,20 @@ use integer;
 use Carp;
 
 
-use constant UTF8 => eval {
+use constant ASCII => eval {
 	require Encode;
+	Encode::find_encoding('ASCII');				# return encoding object
+} || 0;
+
+use constant UTF8 => eval {
 	die if Encode::decode_utf8( chr(91) ) ne '[';		# specifically not UTF-EBCDIC
 	Encode::find_encoding('UTF8');				# return encoding object
-} || undef;
-
-use constant ASCII => eval {
-	Encode::find_encoding('ASCII');				# return encoding object
-} || undef;
+} || 0;
 
 use constant LIBIDN => eval {
 	require Net::LibIDN;					# tested and working
 	UTF8 && Net::LibIDN::idn_to_ascii( pack( 'U*', 20013, 22269 ), 'utf-8' ) eq 'xn--fiqs8s';
-} || undef;
+} || 0;
 
 
 =head1 METHODS
