@@ -2,10 +2,10 @@ package Net::DNS::DomainName;
 use base qw(Net::DNS::Domain);
 
 #
-# $Id: DomainName.pm 954 2011-11-02 21:32:55Z willem $
+# $Id: DomainName.pm 973 2012-01-23 13:33:08Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 954 $)[1];
+$VERSION = (qw$LastChangedRevision: 973 $)[1];
 
 
 =head1 NAME
@@ -182,14 +182,14 @@ sub encode {
 	while (@labels) {
 		my $name = join( '.', @labels );
 
-		return $data . pack( 'n', $hash->{$name} ) if defined $hash->{$name};
+		return $data . pack( 'n', 0xC000 | $hash->{$name} ) if defined $hash->{$name};
 
 		my $label  = shift @labels;
 		my $length = length $label;
 		$data .= pack( 'C a*', $length, $label );
 
 		next unless $offset < 0x4000;
-		$hash->{$name} = 0xC000 | $offset;
+		$hash->{$name} = $offset;
 		$offset += 1 + $length;
 	}
 	$data .= chr(0);
